@@ -1,8 +1,8 @@
 """
-This is an object-orient plotting library.
+This is an object-oriented plotting library.
 
 A procedural interface is provided by the companion pyplot module,
-which may be imported directly, e.g::
+which may be imported directly, e.g.::
 
     from matplotlib.pyplot import *
 
@@ -90,22 +90,51 @@ The base matplotlib namespace includes:
         for the first time.  In particular, it must be called
         **before** importing pylab (if pylab is imported).
 
-matplotlib was initially written by John D. Hunter (jdh2358 at
-gmail.com) and is now developed and maintained by a host of others.
+matplotlib was initially written by John D. Hunter (1968-2012) and is now
+developed and maintained by a host of others.
 
 Occasionally the internal documentation (python docstrings) will refer
 to MATLAB&reg;, a registered trademark of The MathWorks, Inc.
 
 """
-from __future__ import generators
+from __future__ import print_function
 
-__version__  = '1.1.1'
-__version__numpy__ = '1.4' # minimum required numpy version
+import sys
 
-import os, re, shutil, subprocess, sys, warnings
+__version__  = '1.3.0'
+__version__numpy__ = '1.5' # minimum required numpy version
+
+try:
+    import dateutil
+except ImportError:
+    raise ImportError("matplotlib requires dateutil")
+
+try:
+    import pyparsing
+except ImportError:
+    raise ImportError("matplotlib requires pyparsing")
+else:
+    _required = [1, 5, 6]
+    if [int(x) for x in pyparsing.__version__.split('.')] < _required:
+        raise ImportError(
+            "matplotlib requires pyparsing >= {0}".format(
+                '.'.join(str(x) for x in _required)))
+
+import os, re, shutil, warnings
 import distutils.sysconfig
 import distutils.version
 
+# cbook must import matplotlib only within function
+# definitions, so it is safe to import from it here.
+from matplotlib.cbook import is_string_like
+from matplotlib.compat import subprocess
+
+try:
+    reload
+except NameError:
+    # Python 3
+    from imp import reload
+
 # Needed for toolkit setuptools support
 if 0:
     try:
@@ -116,28 +145,26 @@ if 0:
 if not hasattr(sys, 'argv'):  # for modpython
     sys.argv = ['modpython']
 
-"""
-Manage user customizations through a rc file.
-
-The default file location is given in the following order
-
-  - environment variable MATPLOTLIBRC
-
-  - HOME/.matplotlib/matplotlibrc if HOME is defined
-
-  - PATH/matplotlibrc where PATH is the return value of
-    get_data_path()
-"""
 
 import sys, os, tempfile
 
+if sys.version_info[0] >= 3:
+    def ascii(s): return bytes(s, 'ascii')
+
+    def byte2str(b): return b.decode('ascii')
+
+else:
+    ascii = str
+
+    def byte2str(b): return b
+
+
 from matplotlib.rcsetup import (defaultParams,
                                 validate_backend,
-                                validate_toolbar,
-                                validate_cairo_format)
+                                validate_toolbar)
 
 major, minor1, minor2, s, tmp = sys.version_info
-_python24 = major>=2 and minor1>=4
+_python24 = (major == 2 and minor1 >= 4) or major >= 3
 
 # the havedate check was a legacy from old matplotlib which preceeded
 # datetime support
@@ -163,26 +190,33 @@ if not found_version >= expected_version:
 del version
 
 
-def is_string_like(obj):
-    if hasattr(obj, 'shape'): return 0
-    try: obj + ''
-    except (TypeError, ValueError): return 0
-    return 1
-
-
 def _is_writable_dir(p):
     """
     p is a string pointing to a putative writable dir -- return True p
     is such a string, else False
     """
-    try: p + ''  # test is string like
-    except TypeError: return False
+    try:
+        p + ''  # test is string like
+    except TypeError:
+        return False
+
+    # Test whether the operating system thinks it's a writable directory.
+    # Note that this check is necessary on Google App Engine, because the
+    # subsequent check will succeed even though p may not be writable.
+    if not os.access(p, os.W_OK) or not os.path.isdir(p):
+        return False
+
+    # Also test that it is actually possible to write to a file here.
     try:
         t = tempfile.TemporaryFile(dir=p)
-        t.write('1')
-        t.close()
-    except OSError: return False
-    else: return True
+        try:
+            t.write(ascii('1'))
+        finally:
+            t.close()
+    except OSError:
+        return False
+
+    return True
 
 class Verbose:
     """
@@ -197,8 +231,14 @@ class Verbose:
     _commandLineVerbose = None
 
     for arg in sys.argv[1:]:
-        if not arg.startswith('--verbose-'): continue
-        _commandLineVerbose = arg[10:]
+        if not arg.startswith('--verbose-'):
+            continue
+        level_str = arg[10:]
+        # If it doesn't match one of ours, then don't even
+        # bother noting it, we are just a 3rd-party library
+        # to somebody else's script.
+        if level_str in levels:
+            _commandLineVerbose = level_str
 
     def __init__(self):
         self.set_level('silent')
@@ -210,8 +250,10 @@ class Verbose:
         if self._commandLineVerbose is not None:
             level = self._commandLineVerbose
         if level not in self.levels:
-            raise ValueError('Illegal verbose string "%s".  Legal values are %s'%(level, self.levels))
-        self.level = level
+            warnings.warn('matplotlib: unrecognized --verbose-* string "%s".'
+                          ' Legal values are %s' % (level, self.levels))
+        else:
+            self.level = level
 
     def set_fileo(self, fname):
         std = {
@@ -222,7 +264,7 @@ class Verbose:
             self.fileo = std[fname]
         else:
             try:
-                fileo = file(fname, 'w')
+                fileo = open(fname, 'w')
             except IOError:
                 raise ValueError('Verbose object could not open log file "%s" for writing.\nCheck your matplotlibrc verbose.fileo setting'%fname)
             else:
@@ -235,7 +277,7 @@ class Verbose:
 
         """
         if self.ge(level):
-            print >>self.fileo, s
+            print(s, file=self.fileo)
             return True
         return False
 
@@ -268,12 +310,13 @@ class Verbose:
 verbose=Verbose()
 
 
+
 def checkdep_dvipng():
     try:
         s = subprocess.Popen(['dvipng','-version'], stdout=subprocess.PIPE,
                              stderr=subprocess.PIPE)
         line = s.stdout.readlines()[1]
-        v = line.split()[-1]
+        v = byte2str(line.split()[-1])
         return v
     except (IndexError, ValueError, OSError):
         return None
@@ -286,7 +329,7 @@ def checkdep_ghostscript():
             command_args = ['gs', '--version']
         s = subprocess.Popen(command_args, stdout=subprocess.PIPE,
                              stderr=subprocess.PIPE)
-        v = s.stdout.read()[:-1]
+        v = byte2str(s.stdout.read()[:-1])
         return v
     except (IndexError, ValueError, OSError):
         return None
@@ -295,7 +338,7 @@ def checkdep_tex():
     try:
         s = subprocess.Popen(['tex','-version'], stdout=subprocess.PIPE,
                              stderr=subprocess.PIPE)
-        line = s.stdout.readlines()[0]
+        line = byte2str(s.stdout.readlines()[0])
         pattern = '3\.1\d+'
         match = re.search(pattern, line)
         v = match.group(0)
@@ -308,8 +351,8 @@ def checkdep_pdftops():
         s = subprocess.Popen(['pdftops','-v'], stdout=subprocess.PIPE,
                              stderr=subprocess.PIPE)
         for line in s.stderr:
-            if 'version' in line:
-                v = line.split()[-1]
+            if b'version' in line:
+                v = byte2str(line.split()[-1])
         return v
     except (IndexError, ValueError, UnboundLocalError, OSError):
         return None
@@ -319,8 +362,8 @@ def checkdep_inkscape():
         s = subprocess.Popen(['inkscape','-V'], stdout=subprocess.PIPE,
                              stderr=subprocess.PIPE)
         for line in s.stdout:
-            if 'Inkscape' in line:
-                v = line.split()[1]
+            if b'Inkscape' in line:
+                v = byte2str(line.split()[1])
                 break
         return v
     except (IndexError, ValueError, UnboundLocalError, OSError):
@@ -331,8 +374,8 @@ def checkdep_xmllint():
         s = subprocess.Popen(['xmllint','--version'], stdout=subprocess.PIPE,
                              stderr=subprocess.PIPE)
         for line in s.stderr:
-            if 'version' in line:
-                v = line.split()[-1]
+            if b'version' in line:
+                v = byte2str(line.split()[-1])
                 break
         return v
     except (IndexError, ValueError, UnboundLocalError, OSError):
@@ -426,63 +469,135 @@ def checkdep_usetex(s):
 
 def _get_home():
     """Find user's home directory if possible.
-    Otherwise raise error.
+    Otherwise, returns None.
 
-    :see:  http://mail.python.org/pipermail/python-list/2005-February/263921.html
+    :see:  http://mail.python.org/pipermail/python-list/2005-February/325395.html
     """
-    path=''
     try:
-        path=os.path.expanduser("~")
-    except:
+        path = os.path.expanduser("~")
+    except ImportError:
+        # This happens on Google App Engine (pwd module is not present).
         pass
-    if not os.path.isdir(path):
-        for evar in ('HOME', 'USERPROFILE', 'TMP'):
-            try:
-                path = os.environ[evar]
-                if os.path.isdir(path):
-                    break
-            except: pass
-    if path:
-        return path
     else:
-        raise RuntimeError('please define environment variable $HOME')
+        if os.path.isdir(path):
+            return path
+    for evar in ('HOME', 'USERPROFILE', 'TMP'):
+        path = os.environ.get(evar)
+        if path is not None and os.path.isdir(path):
+            return path
+    return None
 
 
+def _create_tmp_config_dir():
+    """
+    If the config directory can not be created, create a temporary
+    directory.
+
+    Returns None if a writable temporary directory could not be created.
+    """
+    import getpass
+    import tempfile
+
+    try:
+        tempdir = tempfile.gettempdir()
+    except NotImplementedError:
+        # Some restricted platforms (such as Google App Engine) do not provide
+        # gettempdir.
+        return None
+    tempdir = os.path.join(tempdir, 'matplotlib-%s' % getpass.getuser())
+    os.environ['MPLCONFIGDIR'] = tempdir
+
+    return tempdir
+
 
 get_home = verbose.wrap('$HOME=%s', _get_home, always=False)
 
-def _get_configdir():
+def _get_xdg_config_dir():
     """
-    Return the string representing the configuration dir.
+    Returns the XDG configuration directory, according to the `XDG
+    base directory spec
+    <http://standards.freedesktop.org/basedir-spec/basedir-spec-latest.html>`_.
+    """
+    return os.environ.get('XDG_CONFIG_HOME', os.path.join(get_home(), '.config'))
 
-    default is HOME/.matplotlib.  you can override this with the
-    MPLCONFIGDIR environment variable
+
+def _get_xdg_cache_dir():
     """
+    Returns the XDG cache directory, according to the `XDG
+    base directory spec
+    <http://standards.freedesktop.org/basedir-spec/basedir-spec-latest.html>`_.
+    """
+    return os.environ.get('XDG_CACHE_HOME', os.path.join(get_home(), '.cache'))
+
+
+def _get_config_or_cache_dir(xdg_base):
+    from matplotlib.cbook import mkdirs
 
     configdir = os.environ.get('MPLCONFIGDIR')
     if configdir is not None:
         if not os.path.exists(configdir):
-            os.makedirs(configdir)
+            from matplotlib.cbook import mkdirs
+            mkdirs(configdir)
+
         if not _is_writable_dir(configdir):
-            raise RuntimeError('Could not write to MPLCONFIGDIR="%s"'%configdir)
+            return _create_tmp_config_dir()
         return configdir
 
     h = get_home()
-    p = os.path.join(get_home(), '.matplotlib')
+    p = os.path.join(h, '.matplotlib')
+    if (sys.platform.startswith('linux') and
+        not os.path.exists(p)):
+        p = os.path.join(xdg_base, 'matplotlib')
 
     if os.path.exists(p):
         if not _is_writable_dir(p):
-            raise RuntimeError("'%s' is not a writable dir; you must set %s/.matplotlib to be a writable dir.  You can also set environment variable MPLCONFIGDIR to any writable directory where you want matplotlib data stored "% (h, h))
+            return _create_tmp_config_dir()
     else:
-        if not _is_writable_dir(h):
-            raise RuntimeError("Failed to create %s/.matplotlib; consider setting MPLCONFIGDIR to a writable directory for matplotlib configuration data"%h)
-
-        os.mkdir(p)
+        try:
+            mkdirs(p)
+        except OSError:
+            return _create_tmp_config_dir()
 
     return p
+
+
+def _get_configdir():
+    """
+    Return the string representing the configuration directory.
+
+    The directory is chosen as follows:
+
+    1. If the MPLCONFIGDIR environment variable is supplied, choose that.
+
+    2a. On Linux, if `$HOME/.matplotlib` exists, choose that, but warn that
+        that is the old location.  Barring that, follow the XDG specification
+        and look first in `$XDG_CONFIG_HOME`, if defined, or `$HOME/.config`.
+
+    2b. On other platforms, choose `$HOME/.matplotlib`.
+
+    3. If the chosen directory exists and is writable, use that as the
+       configuration directory.
+    4. If possible, create a temporary directory, and use it as the
+       configuration directory.
+    5. A writable directory could not be found or created; return None.
+    """
+    return _get_config_or_cache_dir(_get_xdg_config_dir())
+
 get_configdir = verbose.wrap('CONFIGDIR=%s', _get_configdir, always=False)
 
 
+def _get_cachedir():
+    """
+    Return the location of the cache directory.
+
+    The procedure used to find the directory is the same as for
+    _get_config_dir, except using `$XDG_CONFIG_HOME`/`~/.cache` instead.
+    """
+    return _get_config_or_cache_dir(_get_xdg_cache_dir())
+
+get_cachedir = verbose.wrap('CACHEDIR=%s', _get_cachedir, always=False)
+
+
 def _get_data_path():
     'get the path to matplotlib data'
 
@@ -541,9 +656,9 @@ def get_example_data(fname):
 
 def get_py2exe_datafiles():
     datapath = get_data_path()
-    head, tail = os.path.split(datapath)
+    _, tail = os.path.split(datapath)
     d = {}
-    for root, dirs, files in os.walk(datapath):
+    for root, _, files in os.walk(datapath):
         # Need to explicitly remove cocoa_agg files or py2exe complains
         # NOTE I dont know why, but do as previous version
         if 'Matplotlib.nib' in files:
@@ -552,45 +667,41 @@ def get_py2exe_datafiles():
         root = root.replace(tail, 'mpl-data')
         root = root[root.index('mpl-data'):]
         d[root] = files
-    return d.items()
+    return list(d.items())
 
 
 def matplotlib_fname():
     """
-    Return the path to the rc file
+    Get the location of the config file.
 
-    Search order:
+    The file location is determined in the following order
 
-     * current working dir
-     * environ var MATPLOTLIBRC
-     * HOME/.matplotlib/matplotlibrc
-     * MATPLOTLIBDATA/matplotlibrc
+    - `$PWD/matplotlibrc`
 
+    - environment variable `MATPLOTLIBRC`
 
+    - `$MPLCONFIGDIR/matplotlib`
+
+    - On Linux,
+
+          - `$HOME/.matplotlib/matplotlibrc`, if it exists
+
+          - or `$XDG_CONFIG_HOME/matplotlib/matplotlibrc` (if
+            $XDG_CONFIG_HOME is defined)
+
+          - or `$HOME/.config/matplotlib/matplotlibrc` (if
+            $XDG_CONFIG_HOME is not defined)
+
+    - On other platforms,
+
+         - `$HOME/.matplotlib/matplotlibrc` if `$HOME` is defined.
+
+    - Lastly, it looks in `$MATPLOTLIBDATA/matplotlibrc` for a
+      system-defined copy.
     """
-
-    oldname = os.path.join( os.getcwd(), '.matplotlibrc')
-    if os.path.exists(oldname):
-        print >> sys.stderr, """\
-WARNING: Old rc filename ".matplotlibrc" found in working dir
-  and and renamed to new default rc file name "matplotlibrc"
-  (no leading"dot"). """
-        shutil.move('.matplotlibrc', 'matplotlibrc')
-
-    home = get_home()
-    oldname = os.path.join( home, '.matplotlibrc')
-    if os.path.exists(oldname):
-        configdir = get_configdir()
-        newname = os.path.join(configdir, 'matplotlibrc')
-        print >> sys.stderr, """\
-WARNING: Old rc filename "%s" found and renamed to
-  new default rc file name "%s"."""%(oldname, newname)
-
-        shutil.move(oldname, newname)
-
-
-    fname = os.path.join( os.getcwd(), 'matplotlibrc')
-    if os.path.exists(fname): return fname
+    fname = os.path.join(os.getcwd(), 'matplotlibrc')
+    if os.path.exists(fname):
+        return fname
 
     if 'MATPLOTLIBRC' in os.environ:
         path =  os.environ['MATPLOTLIBRC']
@@ -599,14 +710,28 @@ WARNING: Old rc filename "%s" found and renamed to
             if os.path.exists(fname):
                 return fname
 
-    fname = os.path.join(get_configdir(), 'matplotlibrc')
-    if os.path.exists(fname): return fname
+    configdir = _get_configdir()
+    if configdir is not None:
+        fname = os.path.join(configdir, 'matplotlibrc')
+        if os.path.exists(fname):
+            if (sys.platform.startswith('linux') and
+                fname == os.path.join(
+                    get_home(), '.matplotlib', 'matplotlibrc')):
+                warnings.warn(
+                    "Found matplotlib configuration in ~/.matplotlib/. "
+                    "To conform with the XDG base directory standard, "
+                    "this configuration location has been deprecated "
+                    "on Linux, and the new location is now %r/matplotlib/. "
+                    "Please move your configuration there to ensure that "
+                    "matplotlib will continue to find it in the future." %
+                    _get_xdg_config_dir())
+            return fname
 
-
-    path =  get_data_path() # guaranteed to exist or raise
+    path = get_data_path()  # guaranteed to exist or raise
     fname = os.path.join(path, 'matplotlibrc')
     if not os.path.exists(fname):
         warnings.warn('Could not find matplotlibrc; using defaults')
+
     return fname
 
 
@@ -617,15 +742,11 @@ _deprecated_map = {
     'text.fontweight':  'font.weight',
     'text.fontsize':    'font.size',
     'tick.size' :       'tick.major.size',
-    'svg.embed_char_paths' : 'svg.fonttype'
+    'svg.embed_char_paths' : 'svg.fonttype',
+    'savefig.extension' : 'savefig.format'
     }
 
 _deprecated_ignore_map = {
-    'legend.pad' :       'legend.borderpad',
-    'legend.labelsep' :       'legend.labelspacing',
-    'legend.handlelen' :       'legend.handlelength',
-    'legend.handletextsep' :       'legend.handletextpad',
-    'legend.axespad' :       'legend.borderaxespad',
     }
 
 
@@ -638,8 +759,8 @@ class RcParams(dict):
     :mod:`matplotlib.rcsetup`
     """
 
-    validate = dict([ (key, converter) for key, (default, converter) in \
-                     defaultParams.iteritems() ])
+    validate = dict((key, converter) for key, (default, converter) in
+                    defaultParams.iteritems())
     msg_depr = "%s is deprecated and replaced with %s; please use the latter."
     msg_depr_ignore = "%s is deprecated and ignored. Use %s"
 
@@ -670,11 +791,24 @@ See rcParams.keys() for a list of valid parameters.' % (key,))
             key = alt
         return dict.__getitem__(self, key)
 
+    def __repr__(self):
+        import pprint
+        class_name = self.__class__.__name__
+        indent = len(class_name) + 1
+        repr_split = pprint.pformat(dict(self), indent=1,
+                                    width=80 - indent).split('\n')
+        repr_indented = ('\n' + ' ' * indent).join(repr_split)
+        return '{0}({1})'.format(class_name, repr_indented)
+
+    def __str__(self):
+        return '\n'.join('{0}: {1}'.format(k, v)
+                         for k, v in sorted(self.items()))
+
     def keys(self):
         """
         Return sorted list of keys.
         """
-        k = dict.keys(self)
+        k = list(dict.keys(self))
         k.sort()
         return k
 
@@ -684,38 +818,66 @@ See rcParams.keys() for a list of valid parameters.' % (key,))
         """
         return [self[k] for k in self.keys()]
 
-def rc_params(fail_on_error=False):
-    'Return the default params updated from the values in the rc file'
+    def find_all(self, pattern):
+        """
+        Return the subset of this RcParams dictionary whose keys match,
+        using :func:`re.search`, the given ``pattern``.
 
+        .. note::
+
+            Changes to the returned dictionary are *not* propagated to
+            the parent RcParams dictionary.
+
+        """
+        import re
+        pattern_re = re.compile(pattern)
+        return RcParams((key, value)
+                        for key, value in self.items()
+                        if pattern_re.search(key))
+
+
+def rc_params(fail_on_error=False):
+    """Return a :class:`matplotlib.RcParams` instance from the
+    default matplotlib rc file.
+    """
     fname = matplotlib_fname()
     if not os.path.exists(fname):
         # this should never happen, default in mpl-data should always be found
         message = 'could not find rc file; returning defaults'
-        ret = RcParams([ (key, default) for key, (default, converter) in \
+        ret = RcParams([(key, default) for key, (default, _) in \
                         defaultParams.iteritems() ])
         warnings.warn(message)
         return ret
 
+    return rc_params_from_file(fname, fail_on_error)
+
+
+def rc_params_from_file(fname, fail_on_error=False):
+    """Return a :class:`matplotlib.RcParams` instance from the
+    contents of the given filename.
+    """
     cnt = 0
     rc_temp = {}
-    for line in file(fname):
-        cnt += 1
-        strippedline = line.split('#',1)[0].strip()
-        if not strippedline: continue
-        tup = strippedline.split(':',1)
-        if len(tup) !=2:
-            warnings.warn('Illegal line #%d\n\t%s\n\tin file "%s"'%\
-                          (cnt, line, fname))
-            continue
-        key, val = tup
-        key = key.strip()
-        val = val.strip()
-        if key in rc_temp:
-            warnings.warn('Duplicate key in file "%s", line #%d'%(fname,cnt))
-        rc_temp[key] = (val, line, cnt)
+    with open(fname) as fd:
+        for line in fd:
+            cnt += 1
+            strippedline = line.split('#', 1)[0].strip()
+            if not strippedline: continue
+            tup = strippedline.split(':', 1)
+            if len(tup) != 2:
+                warnings.warn('Illegal line #%d\n\t%s\n\tin file "%s"' % \
+                              (cnt, line, fname))
+                continue
+            key, val = tup
+            key = key.strip()
+            val = val.strip()
+            if key in rc_temp:
+                warnings.warn('Duplicate key in file "%s", line #%d' % \
+                              (fname, cnt))
+            rc_temp[key] = (val, line, cnt)
 
-    ret = RcParams([ (key, default) for key, (default, converter) in \
-                    defaultParams.iteritems() ])
+    ret = RcParams([(key, default) for key, (default, _) in \
+                    defaultParams.iteritems()])
 
     for key in ('verbose.level', 'verbose.fileo'):
         if key in rc_temp:
@@ -724,7 +886,7 @@ def rc_params(fail_on_error=False):
                 ret[key] = val # try to convert to proper type or raise
             else:
                 try: ret[key] = val # try to convert to proper type or skip
-                except Exception, msg:
+                except Exception as msg:
                     warnings.warn('Bad val "%s" on line #%d\n\t"%s"\n\tin file \
 "%s"\n\t%s' % (val, cnt, line, fname, msg))
 
@@ -737,19 +899,19 @@ def rc_params(fail_on_error=False):
                 ret[key] = val # try to convert to proper type or raise
             else:
                 try: ret[key] = val # try to convert to proper type or skip
-                except Exception, msg:
+                except Exception as msg:
                     warnings.warn('Bad val "%s" on line #%d\n\t"%s"\n\tin file \
 "%s"\n\t%s' % (val, cnt, line, fname, msg))
         elif key in _deprecated_ignore_map:
             warnings.warn('%s is deprecated. Update your matplotlibrc to use %s instead.'% (key, _deprecated_ignore_map[key]))
 
         else:
-            print >> sys.stderr, """
+            print("""
 Bad key "%s" on line %d in
 %s.
 You probably need to get an updated matplotlibrc file from
 http://matplotlib.sf.net/_static/matplotlibrc or from the matplotlib source
-distribution""" % (key, cnt, fname)
+distribution""" % (key, cnt, fname), file=sys.stderr)
 
     if ret['datapath'] is None:
         ret['datapath'] = get_data_path()
@@ -799,10 +961,10 @@ if rcParams['axes.formatter.use_locale']:
 
 def rc(group, **kwargs):
     """
-    Set the current rc params.  Group is the grouping for the rc, eg.
+    Set the current rc params.  Group is the grouping for the rc, e.g.,
     for ``lines.linewidth`` the group is ``lines``, for
     ``axes.facecolor``, the group is ``axes``, and so on.  Group may
-    also be a list or tuple of group names, eg. (*xtick*, *ytick*).
+    also be a list or tuple of group names, e.g., (*xtick*, *ytick*).
     *kwargs* is a dictionary attribute name/value pairs, eg::
 
       rc('lines', linewidth=2, color='r')
@@ -833,7 +995,7 @@ def rc(group, **kwargs):
 
 
     Note you can use python's kwargs dictionary facility to store
-    dictionaries of default parameters.  Eg, you can customize the
+    dictionaries of default parameters.  e.g., you can customize the
     font rc as follows::
 
       font = {'family' : 'monospace',
@@ -860,7 +1022,7 @@ def rc(group, **kwargs):
     if is_string_like(group):
         group = (group,)
     for g in group:
-        for k,v in kwargs.items():
+        for k,v in kwargs.iteritems():
             name = aliases.get(k) or k
             key = '%s.%s' % (g, name)
             try:
@@ -871,12 +1033,61 @@ def rc(group, **kwargs):
 
 def rcdefaults():
     """
-    Restore the default rc params - these are not the params loaded by
+    Restore the default rc params.  These are not the params loaded by
     the rc file, but mpl's internal params.  See rc_file_defaults for
     reloading the default params from the rc file
     """
     rcParams.update(rcParamsDefault)
 
+
+def rc_file(fname):
+    """
+    Update rc params from file.
+    """
+    rcParams.update(rc_params_from_file(fname))
+
+
+class rc_context(object):
+    """
+    Return a context manager for managing rc settings.
+
+    This allows one to do::
+
+        with mpl.rc_context(fname='screen.rc'):
+            plt.plot(x, a)
+            with mpl.rc_context(fname='print.rc'):
+                plt.plot(x, b)
+            plt.plot(x, c)
+
+    The 'a' vs 'x' and 'c' vs 'x' plots would have settings from
+    'screen.rc', while the 'b' vs 'x' plot would have settings from
+    'print.rc'.
+
+    A dictionary can also be passed to the context manager::
+
+        with mpl.rc_context(rc={'text.usetex': True}, fname='screen.rc'):
+            plt.plot(x, a)
+
+    The 'rc' dictionary takes precedence over the settings loaded from
+    'fname'.  Passing a dictionary only is also valid.
+    """
+
+    def __init__(self, rc=None, fname=None):
+        self.rcdict = rc
+        self.fname = fname
+        self._rcparams = rcParams.copy()
+        if self.fname:
+            rc_file(self.fname)
+        if self.rcdict:
+            rcParams.update(self.rcdict)
+
+    def __enter__(self):
+        return self
+
+    def __exit__(self, type, value, tb):
+        rcParams.update(self._rcparams)
+
+
 def rc_file_defaults():
     """
     Restore the default rc params from the original matplotlib rc that
@@ -890,17 +1101,15 @@ matplotlib.use() must be called *before* pylab, matplotlib.pyplot,
 or matplotlib.backends is imported for the first time.
 """
 
-def use(arg, warn=True):
+def use(arg, warn=True, force=False):
     """
     Set the matplotlib backend to one of the known backends.
 
-    The argument is case-insensitive.  For the Cairo backend,
-    the argument can have an extension to indicate the type of
-    output.  Example:
-
-        use('cairo.pdf')
-
-    will specify a default of pdf output generated by Cairo.
+    The argument is case-insensitive. *warn* specifies whether a
+    warning should be issued if a backend has already been set up.
+    *force* is an **experimental** flag that tells matplotlib to
+    attempt to initialize a new backend by reloading the backend
+    module.
 
     .. note::
 
@@ -909,33 +1118,45 @@ def use(arg, warn=True):
         before importing matplotlib.backends.  If warn is True, a warning
         is issued if you try and call this after pylab or pyplot have been
         loaded.  In certain black magic use cases, e.g.
-        :func:`pyplot.switch_backends`, we are doing the reloading necessary to
-        make the backend switch work (in some cases, e.g. pure image
-        backends) so one can set warn=False to supporess the warnings.
+        :func:`pyplot.switch_backend`, we are doing the reloading necessary to
+        make the backend switch work (in some cases, e.g., pure image
+        backends) so one can set warn=False to suppress the warnings.
 
     To find out which backend is currently set, see
     :func:`matplotlib.get_backend`.
 
     """
-    if 'matplotlib.backends' in sys.modules:
-        if warn: warnings.warn(_use_error_msg)
-        return
+    # Lets determine the proper backend name first
     if arg.startswith('module://'):
         name = arg
     else:
         # Lowercase only non-module backend names (modules are case-sensitive)
         arg = arg.lower()
-        be_parts = arg.split('.')
-        name = validate_backend(be_parts[0])
-        if len(be_parts) > 1:
-            if name == 'cairo':
-                rcParams['cairo.format'] = validate_cairo_format(be_parts[1])
-            else:
-                raise ValueError('Only cairo backend has a format option')
+        name = validate_backend(arg)
+
+    # Check if we've already set up a backend
+    if 'matplotlib.backends' in sys.modules:
+        # Warn only if called with a different name
+        if (rcParams['backend'] != name) and warn:
+            warnings.warn(_use_error_msg)
+
+        # Unless we've been told to force it, just return
+        if not force:
+            return
+        need_reload = True
+    else:
+        need_reload = False
+
+    # Store the backend name
     rcParams['backend'] = name
 
+    # If needed we reload here because a lot of setup code is triggered on
+    # module import. See backends/__init__.py for more detail.
+    if need_reload:
+        reload(sys.modules['matplotlib.backends'])
+
 def get_backend():
-    "Returns the current backend."
+    """Return the name of the current backend."""
     return rcParams['backend']
 
 def interactive(b):
@@ -974,24 +1195,50 @@ for s in sys.argv[1:]:
 
 default_test_modules = [
     'matplotlib.tests.test_agg',
+    'matplotlib.tests.test_arrow_patches',
+    'matplotlib.tests.test_artist',
+    'matplotlib.tests.test_axes',
+    'matplotlib.tests.test_backend_pdf',
+    'matplotlib.tests.test_backend_pgf',
+    'matplotlib.tests.test_backend_qt4',
     'matplotlib.tests.test_backend_svg',
     'matplotlib.tests.test_basic',
+    'matplotlib.tests.test_bbox_tight',
     'matplotlib.tests.test_cbook',
-    'matplotlib.tests.test_mlab',
-    'matplotlib.tests.test_transforms',
-    'matplotlib.tests.test_axes',
-    'matplotlib.tests.test_figure',
+    'matplotlib.tests.test_coding_standards',
+    'matplotlib.tests.test_collections',
+    'matplotlib.tests.test_colorbar',
+    'matplotlib.tests.test_colors',
+    'matplotlib.tests.test_compare_images',
+    'matplotlib.tests.test_contour',
     'matplotlib.tests.test_dates',
-    'matplotlib.tests.test_spines',
-    'matplotlib.tests.test_image',
-    'matplotlib.tests.test_simplification',
-    'matplotlib.tests.test_mathtext',
-    'matplotlib.tests.test_text',
-    'matplotlib.tests.test_tightlayout',
     'matplotlib.tests.test_delaunay',
-    'matplotlib.tests.test_legend'
+    'matplotlib.tests.test_figure',
+    'matplotlib.tests.test_image',
+    'matplotlib.tests.test_legend',
+    'matplotlib.tests.test_lines',
+    'matplotlib.tests.test_mathtext',
+    'matplotlib.tests.test_mlab',
+    'matplotlib.tests.test_patches',
+    'matplotlib.tests.test_path',
+    'matplotlib.tests.test_patheffects',
+    'matplotlib.tests.test_pickle',
+    'matplotlib.tests.test_png',
+    'matplotlib.tests.test_rcparams',
+    'matplotlib.tests.test_scale',
+    'matplotlib.tests.test_simplification',
+    'matplotlib.tests.test_spines',
+    'matplotlib.tests.test_streamplot',
+    'matplotlib.tests.test_subplots',
+    'matplotlib.tests.test_table',
+    'matplotlib.tests.test_text',
+    'matplotlib.tests.test_ticker',
+    'matplotlib.tests.test_tightlayout',
+    'matplotlib.tests.test_transforms',
+    'matplotlib.tests.test_triangulation',
     ]
 
+
 def test(verbosity=1):
     """run the matplotlib test suite"""
     old_backend = rcParams['backend']
@@ -999,8 +1246,9 @@ def test(verbosity=1):
         use('agg')
         import nose
         import nose.plugins.builtin
-        from testing.noseclasses import KnownFailure
+        from .testing.noseclasses import KnownFailure
         from nose.plugins.manager import PluginManager
+        from nose.plugins import multiprocess
 
         # store the old values before overriding
         plugins = []
@@ -1010,6 +1258,11 @@ def test(verbosity=1):
         manager = PluginManager(plugins=plugins)
         config = nose.config.Config(verbosity=verbosity, plugins=manager)
 
+        # Nose doesn't automatically instantiate all of the plugins in the
+        # child processes, so we have to provide the multiprocess plugin with
+        # a list.
+        multiprocess._instantiate_plugins = [KnownFailure]
+
         success = nose.run( defaultTest=default_test_modules,
                             config=config,
                             )
@@ -1025,4 +1278,4 @@ verbose.report('matplotlib version %s'%__version__)
 verbose.report('verbose.level %s'%verbose.level)
 verbose.report('interactive is %s'%rcParams['interactive'])
 verbose.report('platform is %s'%sys.platform)
-verbose.report('loaded modules: %s'%sys.modules.keys(), 'debug')
+verbose.report('loaded modules: %s'%sys.modules.iterkeys(), 'debug')
