@@ -33,6 +33,8 @@ FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 OTHER DEALINGS IN THE SOFTWARE.
 """
 
+from __future__ import print_function
+
 # History:
 # 1.0.10: added float validator (disable "Ok" and "Apply" button when not valid)
 # 1.0.7: added support for "Apply" button
@@ -123,6 +125,19 @@ def text_to_qcolor(text):
     color.setNamedColor(text)
     return color
 
+def is_matplotlib_color(value):
+    """
+    Check if value is a color passed to us from matplotlib.
+    It could either be a valid color string or a 3-tuple of floats between 0. and 1.
+    """
+    if text_to_qcolor(value).isValid():
+        return True
+    if isinstance(value,tuple) and len(value)==3 and all(map(lambda v: isinstance(v,float),value)):
+        for c in value:
+            if c < 0. or c > 1.:
+                return False
+        return True
+    return False
 
 class ColorLayout(QHBoxLayout):
     """Color-specialized QLineEdit layout"""
@@ -237,11 +252,11 @@ class FormWidget(QWidget):
             self.formlayout.addRow(QLabel(comment))
             self.formlayout.addRow(QLabel(" "))
         if DEBUG:
-            print "\n"+("*"*80)
-            print "DATA:", self.data
-            print "*"*80
-            print "COMMENT:", comment
-            print "*"*80
+            print("\n"+("*"*80))
+            print("DATA:", self.data)
+            print("*"*80)
+            print("COMMENT:", comment)
+            print("*"*80)
 
     def get_dialog(self):
         """Return FormDialog instance"""
@@ -253,7 +268,7 @@ class FormWidget(QWidget):
     def setup(self):
         for label, value in self.data:
             if DEBUG:
-                print "value:", value
+                print("value:", value)
             if label is None and value is None:
                 # Separator: (None, None)
                 self.formlayout.addRow(QLabel(" "), QLabel(" "))
@@ -266,11 +281,13 @@ class FormWidget(QWidget):
                 continue
             elif tuple_to_qfont(value) is not None:
                 field = FontLayout(value, self)
-            elif text_to_qcolor(value).isValid():
+            elif is_matplotlib_color(value):
                 field = ColorLayout(QColor(value), self)
             elif isinstance(value, (str, unicode)):
                 field = QLineEdit(value, self)
             elif isinstance(value, (list, tuple)):
+                if isinstance(value, tuple):
+                    value = list(value)
                 selindex = value.pop(0)
                 field = QComboBox(self)
                 if isinstance(value[0], (list, tuple)):
@@ -284,13 +301,13 @@ class FormWidget(QWidget):
                 elif selindex in keys:
                     selindex = keys.index(selindex)
                 elif not isinstance(selindex, int):
-                    print >>STDERR, "Warning: '%s' index is invalid (label: " \
-                                    "%s, value: %s)" % (selindex, label, value)
+                    print("Warning: '%s' index is invalid (label: " \
+                                    "%s, value: %s)" % (selindex, label, value), file=STDERR)
                     selindex = 0
                 field.setCurrentIndex(selindex)
             elif isinstance(value, bool):
                 field = QCheckBox(self)
-                if value :
+                if value:
                     field.setCheckState(Qt.Checked)
                 else :
                     field.setCheckState(Qt.Unchecked)
@@ -325,7 +342,7 @@ class FormWidget(QWidget):
                 continue
             elif tuple_to_qfont(value) is not None:
                 value = field.get_font()
-            elif isinstance(value, (str, unicode)):
+            elif isinstance(value, (str, unicode)) or is_matplotlib_color(value):
                 value = unicode(field.text())
             elif isinstance(value, (list, tuple)):
                 index = int(field.currentIndex())
@@ -336,7 +353,7 @@ class FormWidget(QWidget):
             elif isinstance(value, bool):
                 value = field.checkState() == Qt.Checked
             elif isinstance(value, float):
-                value = float(field.text())
+                value = float(str(field.text()))
             elif isinstance(value, int):
                 value = int(field.value())
             elif isinstance(value, datetime.datetime):
@@ -504,10 +521,9 @@ def fedit(data, title="", comment="", icon=None, parent=None, apply=None):
     """
 
     # Create a QApplication instance if no instance currently exists
-    # (e.g. if the module is used directly from the interpreter)
+    # (e.g., if the module is used directly from the interpreter)
     if QApplication.startingUp():
         _app = QApplication([])
-
     dialog = FormDialog(data, title, comment, icon, parent, apply)
     if dialog.exec_():
         return dialog.get()
@@ -541,19 +557,19 @@ if __name__ == "__main__":
     #--------- datalist example
     datalist = create_datalist_example()
     def apply_test(data):
-        print "data:", data
-    print "result:", fedit(datalist, title="Example",
+        print("data:", data)
+    print("result:", fedit(datalist, title="Example",
                            comment="This is just an <b>example</b>.",
-                           apply=apply_test)
+                           apply=apply_test))
 
     #--------- datagroup example
     datagroup = create_datagroup_example()
-    print "result:", fedit(datagroup, "Global title")
+    print("result:", fedit(datagroup, "Global title"))
 
     #--------- datagroup inside a datagroup example
     datalist = create_datalist_example()
     datagroup = create_datagroup_example()
-    print "result:", fedit(((datagroup, "Title 1", "Tab 1 comment"),
+    print("result:", fedit(((datagroup, "Title 1", "Tab 1 comment"),
                             (datalist, "Title 2", "Tab 2 comment"),
                             (datalist, "Title 3", "Tab 3 comment")),
-                            "Global title")
+                            "Global title"))
