@@ -1,17 +1,26 @@
 #=======================================================================
+
 """ A set of utilities for comparing results.
 """
 #=======================================================================
 
+from __future__ import division
+
 import matplotlib
+from matplotlib.compat import subprocess
 from matplotlib.testing.noseclasses import ImageComparisonFailure
+from matplotlib.testing import image_util
+from matplotlib import _png
+from matplotlib import _get_cachedir
+from matplotlib import cbook
+from distutils import version
+import hashlib
 import math
-import operator
 import os
 import numpy as np
 import shutil
-import subprocess
 import sys
+from functools import reduce
 
 #=======================================================================
 
@@ -22,6 +31,15 @@ __all__ = [
           ]
 
 #-----------------------------------------------------------------------
+
+def make_test_filename(fname, purpose):
+    """
+    Make a new filename by inserting `purpose` before the file's
+    extension.
+    """
+    base, ext = os.path.splitext(fname)
+    return '%s-%s%s' % (base, purpose, ext)
+
 def compare_float( expected, actual, relTol = None, absTol = None ):
    """Fail if the floating point values are not close enough, with
       the givem message.
@@ -32,7 +50,7 @@ def compare_float( expected, actual, relTol = None, absTol = None ):
       exMsg = "You haven't specified a 'relTol' relative tolerance "
       exMsg += "or a 'absTol' absolute tolerance function argument.  "
       exMsg += "You must specify one."
-      raise ValueError, exMsg
+      raise ValueError(exMsg)
 
    msg = ""
 
@@ -81,35 +99,59 @@ def compare_float( expected, actual, relTol = None, absTol = None ):
 # A dictionary that maps filename extensions to functions that map
 # parameters old and new to a list that can be passed to Popen to
 # convert files with that extension to png format.
+def get_cache_dir():
+   cachedir = _get_cachedir()
+   if cachedir is None:
+      raise RuntimeError('Could not find a suitable configuration directory')
+   cache_dir = os.path.join(cachedir, 'test_cache')
+   if not os.path.exists(cache_dir):
+      try:
+         cbook.mkdirs(cache_dir)
+      except IOError:
+         return None
+   if not os.access(cache_dir, os.W_OK):
+      return None
+   return cache_dir
+
+def get_file_hash(path, block_size=2**20):
+   md5 = hashlib.md5()
+   with open(path, 'rb') as fd:
+      while True:
+         data = fd.read(block_size)
+         if not data:
+            break
+         md5.update(data)
+   return md5.hexdigest()
+
 converter = { }
 
 def make_external_conversion_command(cmd):
-   def convert(*args):
-      cmdline = cmd(*args)
-      oldname, newname = args
+   def convert(old, new):
+      cmdline = cmd(old, new)
       pipe = subprocess.Popen(cmdline, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
       stdout, stderr = pipe.communicate()
       errcode = pipe.wait()
-      if not os.path.exists(newname) or errcode:
+      if not os.path.exists(new) or errcode:
          msg = "Conversion command failed:\n%s\n" % ' '.join(cmdline)
          if stdout:
             msg += "Standard output:\n%s\n" % stdout
          if stderr:
             msg += "Standard error:\n%s\n" % stderr
-         raise IOError, msg
+         raise IOError(msg)
+
    return convert
 
 if matplotlib.checkdep_ghostscript() is not None:
-   # FIXME: make checkdep_ghostscript return the command
-   if sys.platform == 'win32':
-      gs = 'gswin32c'
-   else:
-      gs = 'gs'
-   cmd = lambda old, new: \
-       [gs, '-q', '-sDEVICE=png16m', '-dNOPAUSE', '-dBATCH',
+    if sys.platform == 'win32':
+        gs = 'gswin32c'
+    else:
+        gs = 'gs'
+
+    cmd = lambda old, new: \
+        [gs, '-q', '-sDEVICE=png16m', '-dNOPAUSE', '-dBATCH',
         '-sOutputFile=' + new, old]
-   converter['pdf'] = make_external_conversion_command(cmd)
-   converter['eps'] = make_external_conversion_command(cmd)
+    converter['pdf'] = make_external_conversion_command(cmd)
+    converter['eps'] = make_external_conversion_command(cmd)
 
 if matplotlib.checkdep_inkscape() is not None:
    cmd = lambda old, new: \
@@ -121,22 +163,45 @@ def comparable_formats():
    on this system.'''
    return ['png'] + converter.keys()
 
-def convert(filename):
+def convert(filename, cache):
    '''
-   Convert the named file into a png file.
-   Returns the name of the created file.
+   Convert the named file into a png file.  Returns the name of the
+   created file.
+
+   If *cache* is True, the result of the conversion is cached in
+   `~/.matplotlib/test_cache/`.  The caching is based on a hash of the
+   exact contents of the input file.  The is no limit on the size of
+   the cache, so it may need to be manually cleared periodically.
    '''
    base, extension = filename.rsplit('.', 1)
    if extension not in converter:
-      raise ImageComparisonFailure, "Don't know how to convert %s files to png" % extension
+      raise ImageComparisonFailure("Don't know how to convert %s files to png" % extension)
    newname = base + '_' + extension + '.png'
    if not os.path.exists(filename):
-      raise IOError, "'%s' does not exist" % filename
+      raise IOError("'%s' does not exist" % filename)
+
    # Only convert the file if the destination doesn't already exist or
    # is out of date.
    if (not os.path.exists(newname) or
        os.stat(newname).st_mtime < os.stat(filename).st_mtime):
+      if cache:
+         cache_dir = get_cache_dir()
+      else:
+         cache_dir = None
+
+      if cache_dir is not None:
+         hash = get_file_hash(filename)
+         new_ext = os.path.splitext(newname)[1]
+         cached_file = os.path.join(cache_dir, hash + new_ext)
+         if os.path.exists(cached_file):
+            shutil.copyfile(cached_file, newname)
+            return newname
+
       converter[extension](filename, newname)
+
+      if cache_dir is not None:
+         shutil.copyfile(newname, cached_file)
+
    return newname
 
 verifiers = { }
@@ -146,7 +211,7 @@ def verify(filename):
    Verify the file through some sort of verification tool.
    """
    if not os.path.exists(filename):
-      raise IOError, "'%s' does not exist" % filename
+      raise IOError("'%s' does not exist" % filename)
    base, extension = filename.rsplit('.', 1)
    verifier = verifiers.get(extension, None)
    if verifier is not None:
@@ -160,7 +225,7 @@ def verify(filename):
             msg += "Standard output:\n%s\n" % stdout
          if stderr:
             msg += "Standard error:\n%s\n" % stderr
-         raise IOError, msg
+         raise IOError(msg)
 
 # Turning this off, because it seems to cause multiprocessing issues
 if matplotlib.checkdep_xmllint() and False:
@@ -171,11 +236,31 @@ def crop_to_same(actual_path, actual_image, expected_path, expected_image):
    # clip the images to the same size -- this is useful only when
    # comparing eps to pdf
    if actual_path[-7:-4] == 'eps' and expected_path[-7:-4] == 'pdf':
-      aw, ah = actual_image.size
-      ew, eh = expected_image.size
-      actual_image = actual_image.crop((aw/2-ew/2, ah/2-eh/2, aw/2+ew/2, ah/2+eh/2))
+      aw, ah = actual_image.shape
+      ew, eh = expected_image.shape
+      actual_image = actual_image[int(aw/2-ew/2):int(aw/2+ew/2),int(ah/2-eh/2):int(ah/2+eh/2)]
    return actual_image, expected_image
 
+def calculate_rms(expectedImage, actualImage):
+   # calculate the per-pixel errors, then compute the root mean square error
+   num_values = np.prod(expectedImage.shape)
+   abs_diff_image = abs(expectedImage - actualImage)
+
+   # On Numpy 1.6, we can use bincount with minlength, which is much faster than
+   # using histogram
+   expected_version = version.LooseVersion("1.6")
+   found_version = version.LooseVersion(np.__version__)
+   if found_version >= expected_version:
+      histogram = np.bincount(abs_diff_image.ravel(), minlength=256)
+   else:
+      histogram = np.histogram(abs_diff_image, bins=np.arange(257))[0]
+
+   sum_of_squares = np.sum(histogram * np.arange(len(histogram))**2)
+   rms = np.sqrt(float(sum_of_squares) / num_values)
+
+   return rms
+
+
 def compare_images( expected, actual, tol, in_decorator=False ):
    '''Compare two image files - not the greatest, but fast and good enough.
 
@@ -189,62 +274,80 @@ def compare_images( expected, actual, tol, in_decorator=False ):
    = INPUT VARIABLES
    - expected  The filename of the expected image.
    - actual    The filename of the actual image.
-   - tol       The tolerance (a unitless float).  This is used to
-               determine the 'fuzziness' to use when comparing images.
+   - tol       The tolerance (a color value difference, where 255 is the
+               maximal difference).  The test fails if the average pixel
+               difference is greater than this value.
    - in_decorator If called from image_comparison decorator, this should be
                True. (default=False)
    '''
 
-   try:
-      from PIL import Image, ImageOps, ImageFilter
-   except ImportError, e:
-      msg = "Image Comparison requires the Python Imaging Library to " \
-            "be installed.  To run tests without using PIL, then use " \
-            "the '--without-tag=PIL' command-line option.\n"           \
-            "Importing PIL failed with the following error:\n%s" % e
-      if in_decorator:
-         raise NotImplementedError, e
-      else:
-         return msg
-
    verify(actual)
 
    # Convert the image to png
    extension = expected.split('.')[-1]
+
+   if not os.path.exists(expected):
+       raise IOError('Baseline image %r does not exist.' % expected)
+
    if extension != 'png':
-      actual = convert(actual)
-      expected = convert(expected)
+      actual = convert(actual, False)
+      expected = convert(expected, True)
 
    # open the image files and remove the alpha channel (if it exists)
-   expectedImage = Image.open( expected ).convert("RGB")
-   actualImage = Image.open( actual ).convert("RGB")
+   expectedImage = _png.read_png_int( expected )
+   actualImage = _png.read_png_int( actual )
+   expectedImage = expectedImage[:, :, :3]
+   actualImage = actualImage[:, :, :3]
 
    actualImage, expectedImage = crop_to_same(actual, actualImage, expected, expectedImage)
 
-   # normalize the images
-   expectedImage = ImageOps.autocontrast( expectedImage, 2 )
-   actualImage = ImageOps.autocontrast( actualImage, 2 )
+   # convert to signed integers, so that the images can be subtracted without
+   # overflow
+   expectedImage = expectedImage.astype(np.int16)
+   actualImage = actualImage.astype(np.int16)
 
    # compare the resulting image histogram functions
-   h1 = expectedImage.histogram()
-   h2 = actualImage.histogram()
-   rms = math.sqrt( reduce(operator.add, map(lambda a,b: (a-b)**2, h1, h2)) / len(h1) )
+   expected_version = version.LooseVersion("1.6")
+   found_version = version.LooseVersion(np.__version__)
 
-   diff_image = os.path.join(os.path.dirname(actual),
-                             'failed-diff-'+os.path.basename(actual))
-   expected_copy = 'expected-'+os.path.basename(actual)
+   # On Numpy 1.6, we can use bincount with minlength, which is much faster than
+   # using histogram
+   if found_version >= expected_version:
+      rms = 0
 
-   if ( (rms / 10000.0) <= tol ):
+      for i in xrange(0, 3):
+         h1p = expectedImage[:,:,i]
+         h2p = actualImage[:,:,i]
+
+         h1h = np.bincount(h1p.ravel(), minlength=256)
+         h2h = np.bincount(h2p.ravel(), minlength=256)
+
+         rms += np.sum(np.power((h1h-h2h), 2))
+   else:
+      rms = 0
+      ns = np.arange(257)
+
+      for i in xrange(0, 3):
+         h1p = expectedImage[:,:,i]
+         h2p = actualImage[:,:,i]
+
+         h1h = np.histogram(h1p, bins=ns)[0]
+         h2h = np.histogram(h2p, bins=ns)[0]
+
+         rms += np.sum(np.power((h1h-h2h), 2))
+
+   rms = calculate_rms(expectedImage, actualImage)
+
+   diff_image = make_test_filename(actual, 'failed-diff')
+
+   if rms <= tol:
       if os.path.exists(diff_image):
          os.unlink(diff_image)
-      if os.path.exists(expected_copy):
-         os.unlink(expected_copy)
       return None
 
    save_diff_image( expected, actual, diff_image )
 
    if in_decorator:
-      shutil.copyfile( expected, expected_copy )
       results = dict(
          rms = rms,
          expected = str(expected),
@@ -253,12 +356,9 @@ def compare_images( expected, actual, tol, in_decorator=False ):
          )
       return results
    else:
-      # expected_copy is only for in_decorator case
-      if os.path.exists(expected_copy):
-         os.unlink(expected_copy)
       # old-style call from mplTest directory
       msg = "  Error: Image files did not match.\n"       \
-            "  RMS Value: " + str( rms / 10000.0 ) + "\n" \
+            "  RMS Value: " + str( rms ) + "\n" \
             "  Expected:\n    " + str( expected ) + "\n"  \
             "  Actual:\n    " + str( actual ) + "\n"      \
             "  Difference:\n    " + str( diff_image ) + "\n"      \
@@ -266,17 +366,28 @@ def compare_images( expected, actual, tol, in_decorator=False ):
       return msg
 
 def save_diff_image( expected, actual, output ):
-   from PIL import Image
-   expectedImage = Image.open( expected ).convert("RGB")
-   actualImage = Image.open( actual ).convert("RGB")
+   expectedImage = _png.read_png( expected )
+   actualImage = _png.read_png( actual )
    actualImage, expectedImage = crop_to_same(actual, actualImage, expected, expectedImage)
    expectedImage = np.array(expectedImage).astype(np.float)
    actualImage = np.array(actualImage).astype(np.float)
    assert expectedImage.ndim==actualImage.ndim
    assert expectedImage.shape==actualImage.shape
    absDiffImage = abs(expectedImage-actualImage)
+
    # expand differences in luminance domain
-   absDiffImage *= 10
-   save_image_np = np.clip(absDiffImage,0,255).astype(np.uint8)
-   save_image = Image.fromarray(save_image_np)
-   save_image.save(output)
+   absDiffImage *= 255 * 10
+   save_image_np = np.clip(absDiffImage, 0, 255).astype(np.uint8)
+   height, width, depth = save_image_np.shape
+
+   # The PDF renderer doesn't produce an alpha channel, but the
+   # matplotlib PNG writer requires one, so expand the array
+   if depth == 3:
+      with_alpha = np.empty((height, width, 4), dtype=np.uint8)
+      with_alpha[:,:,0:3] = save_image_np
+      save_image_np = with_alpha
+
+   # Hard-code the alpha channel to fully solid
+   save_image_np[:,:,3] = 255
+
+   _png.write_png(save_image_np.tostring(), width, height, output)
